==> web_frontend_binect-0.0.55.gem: app/controllers/web_frontend_binect/payments_controller.rb <==
require_dependency "web_frontend_binect/application_controller"
require 'digest'

CONFIG      = Rails.application.config.payment_config
SECRET_KEY  = CONFIG[:secret_key]
CUSTOMER_ID = CONFIG[:customerid]
PORTALID    = CONFIG[:portalid]
AID         = CONFIG[:aid]
EMAIL       = CONFIG[:email]

module WebFrontendBinect
    class PaymentsController < ApplicationController
        include WebFrontendBinect::DocumentBasicHelper
        include RethinkDB::Shortcuts

        skip_before_action  :verify_authenticity_token
        before_action       :set_payment, only: [:show, :edit, :update, :destroy]

        layout :compute_layout

        # GET /payments
        def index
        end

        def payment_success
            reference = params[:ref]
            payment = Payment.find(reference)
            fail "Cannot find payment" if payment.nil?

            @reference = payment.id
            if payment.is_payment_confirmed
                @real_path = "/payments/payment_confirmed"
                render "payment_confirmed"
            else
                @real_path = "/payments/payment_wait"
                render "payment_wait"
            end
        end

        def payment_notification
            Rails.logger.info "** Received payment notification: #{params}"

            reference = params[:reference]
            txaction  = params[:txaction]
            userid    = params[:userid]
            txid      = params[:txid]
            #key       = params[:key]
            email     = params[:email]

            # TODO: Validate key

            fail "Cannot find payment" if reference.nil?
            payment = Payment.find(reference)
            payment.email_address = email if email != EMAIL

            # Payment is not done. We just got the information that it starts.
            if txaction == "appointed"
                # All documents get this state..
                conn = NoBrainer.connection
                r.table('nukleus__documents').get_all(r.args(payment.raw_attributes[:referenced_document_ids])).update({:is_appointed => true}).run(conn.raw)

                payment.payone_user_id = userid
                payment.payone_txid = txid
                payment.is_payment_confirmed = true

            elsif txaction == "paid" && payment.email_address.present?
                conn = NoBrainer.connection
                # All documents get this state..
                r.table('nukleus__documents').get_all(r.args(payment.raw_attributes[:referenced_document_ids])).update({:is_payed => true}).run(conn.raw)

                # Now send a mail to the user in order to tell him that the order will be processed.
                PaymentMailer.send_payment_success(payment).deliver_now
            elsif txaction == "cancelation" && payment.email_address.present?
                # All documents get this state..
                conn = NoBrainer.connection
                r.table('nukleus__documents').get_all(r.args(payment.raw_attributes[:referenced_document_ids])).update({:is_payed => false}).run(conn.raw)

                # Now send a mail to the user in order to tell him that the order is canceled
                PaymentMailer.send_payment_cancel(payment).deliver_now
            end
            payment.save
        end

        def payment_fields
            payment = Payment.new(referenced_document_ids: all_documents(false).map { |d| d.id } )

            # Connect this payment either with the session or with the user id.
            # The session id is necessary if the user is not logged in and will do this afterwards.
            if current_user.present?
                payment.user_id = current_user.id
            else
                payment.session_id = session[:session_id]
            end
            payment.save

            secret_key =                SECRET_KEY

            # The following fields are added to the hash
            payment_params = {
                customerid:             CUSTOMER_ID,
                portalid:               PORTALID,
                aid:                    AID,
                mode:                   "live",
                request:                "authorization",
                reference:              payment.id,
                amount:                 (payment.price_after_tax * 100).to_i,                # in cent
                currency:               "EUR",
                api_version:            "3.8",
                encoding:               "UTF-8",
                invoiceid:              "123456", # TODO: Add Invoice ID?
                invoiceappendix:        "Briefversand ueber Binect.de",
                it:                     "goods",
                id:                     "1-1",
                pr:                     (payment.price_after_tax * 100).to_i,
                no:                     1,
                de:                     "Versand von #{count_all_documents(false)} Briefe(n) ueber Binect.de",
                va:                     19,
                successurl:             "#{request.protocol}#{request.host_with_port}/payments/payment_success/#{payment.id}",
                backurl:                "#{request.protocol}#{request.host_with_port}/payments"
            }

            case params[:payment_type]
            when "optionPaypal"
                payment_params[:clearingtype] = "wlt"
            when "optionCreditCard"
                payment_params[:clearingtype] = "cc"
            when "optionDirect"
                payment_params[:clearingtype] = "sb"
            when "optionAdvance"
                payment_params[:clearingtype] = "vor"
            end

            @sorted_hash = payment_params.sort

            @sorted_hash.each_with_index do |a,i|
                @sorted_hash[i] = ["#{a[0].to_s}[1]",@sorted_hash[i][1]] if [:it,:id,:pr,:no,:de,:va].include? a[0]
            end

            @sorted_hash = @sorted_hash.to_h
            hash_string = ""
            @sorted_hash.each { |k,v| hash_string = "#{hash_string}#{v}" }

            md5 = Digest::MD5.new
            @sorted_hash[:hash] = md5.hexdigest("#{hash_string}#{secret_key}")

            # Set language of payment form
            @sorted_hash[:language]     = "de"
            @sorted_hash[:firstname]    = "Vorname"
            @sorted_hash[:lastname]     = "Nachname"
            @sorted_hash[:company]      = "Firmenname"
            @sorted_hash[:street]       = "Strasse 1"
            @sorted_hash[:zip]          = "64331"
            @sorted_hash[:city]         = "Stadt"
            @sorted_hash[:country]      = "DE"
            if current_user.present?
                @sorted_hash[:email]        = current_user.email
            else
                @sorted_hash[:email]        = EMAIL
            end

            render :payment_fields, :layout => false
        end

        def show_receipt
            @payment = Payment.find(params[:ref])
        end

        private
        def compute_layout
            action_name == "show_receipt" ? "web_frontend_binect/receipt" : "web_frontend_binect/application"
        end

        # Use callbacks to share common setup or constraints between actions.
        def set_payment
            # TODO: make it here..
            #@payment = Payment.find(params[:id])
        end

        # Only allow a trusted parameter "white list" through.
        def payment_params
            params[:payment]
        end
    end
end

==> web_frontend_binect-0.0.65.gem: app/controllers/web_frontend_binect/payments_controller.rb <==
require_dependency "web_frontend_binect/application_controller"
require 'digest'

CONFIG      = Rails.application.config.payment_config
SECRET_KEY  = CONFIG[:secret_key]
CUSTOMER_ID = CONFIG[:customerid]
PORTALID    = CONFIG[:portalid]
AID         = CONFIG[:aid]
EMAIL       = CONFIG[:email]

module WebFrontendBinect
    class PaymentsController < ApplicationController
        include WebFrontendBinect::DocumentBasicHelper
        include RethinkDB::Shortcuts

        skip_before_action  :verify_authenticity_token
        before_action       :set_payment, only: [:show, :edit, :update, :destroy]

        layout :compute_layout

        # GET /payments
        def index
        end

        def payment_success
            reference = params[:ref]
            payment = Payment.find(reference)
            fail "Cannot find payment" if payment.nil?

            @reference = payment.id
            if payment.is_payment_confirmed
                @real_path = "/payments/payment_confirmed"
                render "payment_confirmed"
            else
                @real_path = "/payments/payment_wait"
                render "payment_wait"
            end
        end

        def payment_notification
            Rails.logger.info "** Received payment notification: #{params}"

            reference = params[:reference]
            txaction  = params[:txaction]
            userid    = params[:userid]
            txid      = params[:txid]
            #key       = params[:key]
            email     = params[:email]

            # TODO: Validate key

            fail "Cannot find payment" if reference.nil?
            payment = Payment.find(reference)
            payment.email_address = email if email != EMAIL

            # Payment is not done. We just got the information that it starts.
            if txaction == "appointed"
                # All documents get this state..
                conn = NoBrainer.connection
                r.table('nukleus__documents').get_all(r.args(payment.raw_attributes[:referenced_document_ids])).update({:is_appointed => true}).run(conn.raw)

                payment.payone_user_id = userid
                payment.payone_txid = txid
                payment.is_payment_confirmed = true

            elsif txaction == "paid" && payment.email_address.present?
                conn = NoBrainer.connection
                # All documents get this state..
                r.table('nukleus__documents').get_all(r.args(payment.raw_attributes[:referenced_document_ids])).update({:is_payed => true}).run(conn.raw)

                # Now send a mail to the user in order to tell him that the order will be processed.
                PaymentMailer.send_payment_success(payment).deliver_now
            elsif txaction == "cancelation" && payment.email_address.present?
                # All documents get this state..
                conn = NoBrainer.connection
                r.table('nukleus__documents').get_all(r.args(payment.raw_attributes[:referenced_document_ids])).update({:is_payed => false}).run(conn.raw)

                # Now send a mail to the user in order to tell him that the order is canceled
                PaymentMailer.send_payment_cancel(payment).deliver_now
            end
            payment.save
        end

        def payment_fields
            payment = Payment.new(referenced_document_ids: all_documents(false).delete_if { |d| d.binect_order_id.blank? }.map { |d| d.id } )

            # Connect this payment either with the session or with the user id.
            # The session id is necessary if the user is not logged in and will do this afterwards.
            if current_user.present?
                payment.user_id = current_user.id
            else
                payment.session_id = session[:session_id]
            end
            payment.save

            secret_key =                SECRET_KEY

            # The following fields are added to the hash
            payment_params = {
                customerid:             CUSTOMER_ID,
                portalid:               PORTALID,
                aid:                    AID,
                mode:                   "live",
                request:                "authorization",
                reference:              payment.id,
                amount:                 (payment.price_after_tax * 100).to_i,                # in cent
                currency:               "EUR",
                api_version:            "3.8",
                encoding:               "UTF-8",
                invoiceid:              "123456", # TODO: Add Invoice ID?
                invoiceappendix:        "Briefversand ueber Binect.de",
                it:                     "goods",
                id:                     "1-1",
                pr:                     (payment.price_after_tax * 100).to_i,
                no:                     1,
                de:                     "Versand von #{count_all_documents(false)} Briefe(n) ueber Binect.de",
                va:                     19,
                successurl:             "#{request.protocol}#{request.host_with_port}/payments/payment_success/#{payment.id}",
                backurl:                "#{request.protocol}#{request.host_with_port}/payments"
            }

            case params[:payment_type]
            when "optionPaypal"
                payment_params[:clearingtype] = "wlt"
            when "optionCreditCard"
                payment_params[:clearingtype] = "cc"
            when "optionDirect"
                payment_params[:clearingtype] = "sb"
            when "optionAdvance"
                payment_params[:clearingtype] = "vor"
            end

            @sorted_hash = payment_params.sort

            @sorted_hash.each_with_index do |a,i|
                @sorted_hash[i] = ["#{a[0].to_s}[1]",@sorted_hash[i][1]] if [:it,:id,:pr,:no,:de,:va].include? a[0]
            end

            @sorted_hash = @sorted_hash.to_h
            hash_string = ""
            @sorted_hash.each { |k,v| hash_string = "#{hash_string}#{v}" }

            md5 = Digest::MD5.new
            @sorted_hash[:hash] = md5.hexdigest("#{hash_string}#{secret_key}")

            # Set language of payment form
            @sorted_hash[:language]     = "de"
            @sorted_hash[:firstname]    = "Vorname"
            @sorted_hash[:lastname]     = "Nachname"
            @sorted_hash[:company]      = "Firmenname"
            @sorted_hash[:street]       = "Strasse 1"
            @sorted_hash[:zip]          = "64331"
            @sorted_hash[:city]         = "Stadt"
            @sorted_hash[:country]      = "DE"
            if current_user.present?
                @sorted_hash[:email]        = current_user.email
            else
                @sorted_hash[:email]        = EMAIL
            end

            render :payment_fields, :layout => false
        end

        def show_receipt
            @payment = Payment.find(params[:ref])
        end

        private
        def compute_layout
            action_name == "show_receipt" ? "web_frontend_binect/receipt" : "web_frontend_binect/application"
        end

        # Use callbacks to share common setup or constraints between actions.
        def set_payment
            # TODO: make it here..
            #@payment = Payment.find(params[:id])
        end

        # Only allow a trusted parameter "white list" through.
        def payment_params
            params[:payment]
        end
    end
end
